--[[    
    ROLEPLAY MISSION EDITOR (RME)
    Copyright (C) 2007,2008 Jean RODRIGUEZ  aka Enoril (Ryzom), Enoriel (WoW - Kirin Tor)

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

]]
RME_DebugMsg("<LOAD>RME_Missions.lua",3);

--[[ TOUT CE CODE EST EN COURS DE REECRITURE ]]--

---
--[[ RME_Take_Mission ]]--
	-- Cette fonction permet de valider une mission
	function RME_Take_Mission(MissionID,force)
		RME_DebugMsg("RME_Take_Mission("..tostring(MissionID)..")",4);
		
		if #RME_IN_PROGRESS_MISSIONS_LIST >= RME_MAX_MISSIONS then
			PlaySound("igQuestLogAbandonQuest")
			RME_PlayerNotify(ERR_QUEST_LOG_FULL,0)
			StaticPopup_Show("RME_TOO_MANY_MISSIONS") 
			return
		end
		
		-- On vrifie si l'ID mission existe dans la liste globale
		if not RME_MISSIONS_LIST[MissionID] then
			PlaySound("igQuestLogAbandonQuest")
			RME_PlayerNotify(RME_LOCALE["RME_LOCALE_MISSION_NOT_FOUND"].text,0)
			RME_DebugMsg("RME_Take_Mission("..tostring(MissionID).."): "..RME_LOCALE["RME_LOCALE_MISSION_NOT_FOUND"].text,1);
			return
		end
		
		-- Ok on charge la mission que le joueur veut prendre
		local LOCAL_MISSION = RME_CLASS_MISSION(MissionID)
		LOCAL_MISSION:Load(RME_MISSIONS_LIST[MissionID])
		
		-- On vrifie si cette mission n'a pas dja t termine par le joueur ou si elle n'est pas dja en cours 
		if not LOCAL_MISSION:MissionCanBeTaken() and not force then
			PlaySound("igQuestLogAbandonQuest")
			RME_PlayerNotify(ERR_QUEST_ALREADY_ON.." or "..ERR_QUEST_NEED_PREREQS,0)
			return
		end	

		-- On vrifie, dans le cas d'une mission timer, si il n'y a pas dja une autre mission timer en cours
		if LOCAL_MISSION.MissionTimer and QuestTimerFrame:IsShown() or RME_QuestTimerFrame:IsShown() then
			PlaySound("igQuestLogAbandonQuest")
			RME_PlayerNotify(ERR_QUEST_ONLY_ONE_TIMED,0)
			return
		end

		-- Ok on peut maintenant reellement donner cette mission au joueur
		LOCAL_MISSION:Take_Mission()
		
		-- on place donc la mission dans RME_IN_PROGRESS_MISSIONS_LIST
		RME_IN_PROGRESS_MISSIONS_LIST[LOCAL_MISSION.ID] = LOCAL_MISSION
				
		-- On vrifie les objetifs
		LOCAL_MISSION:CheckObjective()
		
		-- On lance le son: mission accepte
		PlaySound("QUESTADDED")
		RME_PLAYER_HAVE_MISSION_IN_PROGRESS = true
		RME_Mega_Update(true)
	end


--[[ RME_Cancel_Mission ]]--
	-- Cette fonction permet d'abandonner une mission
	-- RME_QUESTLOG_MISSIONID doit contenir l'id de la mission abandonne
	function RME_Cancel_Mission(force)
		RME_DebugMsg("RME_Cancel_Mission("..tostring(force)..")",4);
		
		-- On regarde si une mission est selectionne
		if not RME_QUESTLOG_MISSIONID then
			RME_DebugMsg("RME_Cancel_Mission("..tostring(force).."):No mission selected, skip...",1);
			return 
		end
		
		-- Est ce que cette mission existe dans la liste des missions en cours ?
		if not RME_IN_PROGRESS_MISSIONS_LIST[RME_QUESTLOG_MISSIONID] then
			RME_DebugMsg("RME_Take_Mission("..tostring(MissionID).."): Mission doesn't exist in active missions list",1);
			return
		end
		
		-- Est ce que l'utilisateur a valider l'abandon de la mission ? si non, on affiche un popup
		if not force then
			-- Popup confirmation annulation mission
			StaticPopup_Show("RME_CONFIRM_LEAVE_MISSION") 
			return
		end
		
		-- Ok on abandonne la mission. cela supprime aussi les objets lis a cette mission du sac du joueur
		RME_IN_PROGRESS_MISSIONS_LIST[RME_QUESTLOG_MISSIONID]:Leave_Mission()
		
		-- on supprime cette mission de la liste des missions joueur actives (ds RME_IN_PROGRESS_MISSIONS_LIST)
		RME_IN_PROGRESS_MISSIONS_LIST[RME_QUESTLOG_MISSIONID] = nil
		RME_QUESTLOG_MISSIONID = nil -- on deselectionne la mission supprime
		
		-- On lance le son: mission abandonn
		PlaySound("igQuestLogAbandonQuest")
		
		-- On met a jour l'affichage et les interactions
		RME_Mega_Update(true)
		
		RME_DebugMsg("RME_Cancel_Mission("..tostring(force).."): Done !.",1);
		
	end
	

--[[ RME_Failed_Mission ]]--
	-- Cette fonction permet de marquer une mission comme ayant chou
	function RME_Failed_Mission(MissionID)
		-- On vrifie si l'ID mission existe dans la liste des missions en cours chez le joueur
		if not RME_IN_PROGRESS_MISSIONS_LIST[MissionID] then
			RME_DebugMsg("RME_Failed_Mission("..tostring(MissionID).."): Mission doesn't exist in active missions list",1);
			return
		end
		
		-- Ok on marque la mission comme failed
		RME_IN_PROGRESS_MISSIONS_LIST[MissionID]:Mission_Failed()
		
		-- On lance le son: mission failed
		PlaySound("igQuestFailed")
		-- Mise a jour affichage
		RME_Mega_Update()
	end

	
--[[ RME_Success_Mission ]]--
	-- Cette fonction permet de marquer une mission comme ayant russi
	function RME_Success_Mission(MissionID)
	
		-- On vrifie si l'ID mission existe dans la liste des missions en cours chez le joueur
		if not RME_IN_PROGRESS_MISSIONS_LIST[MissionID] then
			RME_DebugMsg("RME_Success_Mission("..tostring(MissionID).."): Mission doesn't exist in active missions list",1);
			return
		end

		-- Ok on marque la mission comme russi
		RME_IN_PROGRESS_MISSIONS_LIST[MissionID]:Mission_Success()

		-- On lance le son: objectif complete
		PlaySound("QUESTCOMPLETED")
		RME_Mega_Update(true)

	end
	

--[[ RME_Completed_Mission ]]--
	-- Cette fonction permet de marquer une mission comme tant termine
	function RME_Completed_Mission(MissionID,force)
	
		-- On vrifie si l'ID mission existe dans la liste des missions en cours chez le joueur 
		if not RME_IN_PROGRESS_MISSIONS_LIST[MissionID] then
			RME_DebugMsg("RME_Completed_Mission("..tostring(MissionID).."): Mission doesn't exist in active missions list",1);
			return
		end

		-- On vrifie si la mission est bien marqu comme russie
		if not RME_IN_PROGRESS_MISSIONS_LIST[MissionID].MissionSuccess and not force then
			RME_DebugMsg("RME_Completed_Mission("..tostring(MissionID).."): Mission objectives are not done ! Can't completed this mission",1);
			return
		end
			
		-- Ok on marque la mission comme termine et on donne les rcompenses
		RME_IN_PROGRESS_MISSIONS_LIST[MissionID]:Mission_Completed()

		-- On place l'id de cette mission dans la liste des missions completes RME_COMPLETED_MISSIONS_LIST
		RME_COMPLETED_MISSIONS_LIST[MissionID] = MissionID
		
		-- On supprime cette mission de la liste des missions en cours du joueur
		RME_IN_PROGRESS_MISSIONS_LIST[MissionID] = nil
		
		-- On lance le son: mission finie
		PlaySound("QUESTCOMPLETED")	
		
		RME_QUESTLOG_MISSIONID = nil -- on deselectionne la mission termine
		RME_Mega_Update(true)
		
	end

--[[ RME_CheckMissionsTimer ]]--
	function RME_CheckMissionsTimer()
		RME_DebugMsg("RME_CheckMissionsTimer()" ,4);
		local MISSION_ID, LOCAL_MISSION
		for MISSION_ID,LOCAL_MISSION in pairs(RME_IN_PROGRESS_MISSIONS_LIST) do -- LOCAL_MISSION est du style RME_CLASS_MISSION
			RME_DebugMsg("RME_CheckMissionsTimer(): Check Mission ".. tostring(MISSION_ID) ,4);
			-- On check la mission uniquement si elle est en cours, si elle n'est pas fini, si elle n'est pas chou et si elle a un timer en cours (MissionTimerExpiration)
			if LOCAL_MISSION.MissionInProgress and not LOCAL_MISSION.MissionSuccess and not LOCAL_MISSION.MissionFailed and LOCAL_MISSION.MissionTimerExpiration then
				-- Oui ! on fait checker cette mission et on relance cette fonction dans une seconde ou 30
				if not RME_QuestLogFrame:IsShown() then
					RME_QuestTimerFrame.MISSION_ID = LOCAL_MISSION.ID
					RME_QuestTimerFrame:Show()
				end
				
				LOCAL_MISSION:CheckTimer()
				
				if LOCAL_MISSION.MissionTimerExpiration and LOCAL_MISSION.MissionTimerExpiration - time() > 60 then
					RME_EventManager_AddEvent("CHECK_TIMER_MISSION",30,"RME_CheckMissionsTimer()",true)
				else
					RME_EventManager_AddEvent("CHECK_TIMER_MISSION",1,"RME_CheckMissionsTimer()",true)
				end
			end
		end
		if RME_QuestLogFrame:IsShown() then
			RME_QuestLog_UpdateInfo()
		end
	end
--[[ /Gestion mission ]]--

--[[ Fonctions diverses ]]--
	--Ajoute un objet dans le sac
	function RME_AddInBag(LOCAL_ITEM,AddQuantity,Silence)
		RME_DebugMsg("RME_AddInBag()",4);
		-- L'objet ajout existe t il dja dans le sac ? si oui on augment sa quantit
		local	ObjetExist, Quantity, Position = RME_Bag_ItemCount(LOCAL_ITEM)
		if ObjetExist then 
	
			if RME_PLAYER_BAG[Position] and RME_PLAYER_BAG[Position].ID ==  LOCAL_ITEM.ID then --On a trouv un objet similaire dans le sac
				
				-- On met a jour les infos de l'objet EXCEPTE sa quantit qu'on calculera plus bas  (cela permet de mettre a jour les autres infos sans obligatoirement incrmenter sa quantit)
				local tempquantity = RME_PLAYER_BAG[Position].Quantity -- on retient le nombre d'objet actuel dans le sac
				RME_PLAYER_BAG[Position] = LOCAL_ITEM
				RME_PLAYER_BAG[Position].Quantity = tempquantity
				-- Maintenant on gere la quantit
				
				if not LOCAL_ITEM.MaxCarried then
					-- On rajoute la quantit 
					RME_PLAYER_BAG[Position].Quantity = tonumber(Quantity + AddQuantity)					
				else
					-- Il y a une limite d'objet
					if Quantity + LOCAL_ITEM.Quantity > LOCAL_ITEM.MaxCarried then
						-- En rajoutant cet objet on atteint la limite
						-- calcul si on peut en rajouter une partie seulemement
						local New_Quantity = LOCAL_ITEM.MaxCarried - Quantity
						
						if New_Quantity > 0 then
							-- Ok on complte pour atteindre le max autoris
							AddQuantity = New_Quantity
							RME_PLAYER_BAG[Position].Quantity = tonumber(Quantity + AddQuantity)					
						else
							-- On ne peut plus rien rajouter, on ignore cet ajout d'objet
							RME_PlayerNotify(RME_LOCALE["RME_LOCALE_ITEM_CANT_RECEIVE_MORE"].text,0)
							if RME_BagFrame:IsShown() then
								RME_Bag_Update()
							end							
							return
						end
						
					else 
						-- L'objet a une limite de possession mais on ne l'atteint pas en rajoutant cet objet
						-- on le rajoute aux autre simplement
						RME_PLAYER_BAG[Position].Quantity = tonumber(Quantity + AddQuantity)					
					
					end
				end
			end	
			
		else 
			-- Sinon on rajoute simplement l'objet dans le sac au 1er emplacement libre
			for i=1,table.getn(RME_PLAYER_BAG)+1 do		
				if not RME_PLAYER_BAG[i] then --Place libre 				
					-- On rajoute l'objet 
					LOCAL_ITEM.Quantity = tonumber(AddQuantity)
					table.insert(RME_PLAYER_BAG,i,LOCAL_ITEM)
				end
			end
		end
		

		-- on informe le joueur
		if not Silence then
			local r, g, b, hex, Objet
			r, g, b, hex = GetItemQualityColor(LOCAL_ITEM.Quality)	
			Objet = hex.."["..LOCAL_ITEM.Name.."]|r"
			RME_DisplayMessage(format(RME_LOCALE["RME_LOCALE_ITEM_RECEIVE"].text,Objet,AddQuantity))
		end
		-- on update
		if RME_BagFrame:IsShown() then
			RME_Bag_Update()
		end
		RME_Check_Mission_Event()
	end

	-- Retourne le nombre d'objet contenu dans le sac
	function RME_Bag_ItemCount(LOCAL_ITEM)
		
		if type(LOCAL_ITEM) == "table" then
		
			for i=1,table.getn(RME_PLAYER_BAG) do		
				if RME_PLAYER_BAG[i] and RME_PLAYER_BAG[i].ID == LOCAL_ITEM.ID then --On a trouv un objet similaire dans le sac
					-- On rcupere sa position et sa quantit
					return true, tonumber(RME_PLAYER_BAG[i].Quantity), i
				end
			end
		
		elseif type(LOCAL_ITEM) == "string" then
		
			for i=1,table.getn(RME_PLAYER_BAG) do		
				if RME_PLAYER_BAG[i] and RME_PLAYER_BAG[i].ID == LOCAL_ITEM then --On a trouv un objet similaire dans le sac
					-- On rcupere sa position et sa quantit
					return true, tonumber(RME_PLAYER_BAG[i].Quantity), i
				end
			end
		end
		return nil, 0, nil
	end
	
	-- Supprime un objet du sac
	function RME_RemoveInBag(LOCAL_ITEM,RemoveQuantity,Silence)
		RME_DebugMsg("RME_RemoveInBag()",4);
		-- L'objet supprim existe t il dja dans le sac ? si oui on diminue sa quantit
		local ObjetExist, Quantity, Position = RME_Bag_ItemCount(LOCAL_ITEM)
		if not ObjetExist then
			return
		end
		LOCAL_ITEM = RME_PLAYER_BAG[Position]
		if tonumber(tonumber(Quantity) - tonumber(RemoveQuantity)) < 1 then
			-- On supprime l'objet
			table.remove (RME_PLAYER_BAG,Position)
		else
			RME_PLAYER_BAG[Position].Quantity = tonumber(tonumber(Quantity) - tonumber(RemoveQuantity))
		end
		-- On informe le joueur
		if not Silence then
			local r, g, b, hex, Objet
			r, g, b, hex = GetItemQualityColor(LOCAL_ITEM.Quality)
			Objet = hex.."["..LOCAL_ITEM.Name.."]|r"
			if RemoveQuantity > 1 then
				RME_DisplayMessage(format(RME_LOCALE["RME_LOCALE_ITEM_REMOVE"].text,Objet,RemoveQuantity))
			else
				RME_DisplayMessage(format(RME_LOCALE["RME_LOCALE_ITEM_REMOVE_SINGLE"].text,Objet))
			end
		end
		-- on update
		if RME_BagFrame:IsShown() then
			RME_Bag_Update()
		end
		RME_Check_Mission_Event()
	end
	
	-- function desactive un objet dans le sac
	function RME_Bag_DesactivateItem(LOCAL_ITEM_ID,SetDesactivated)
		RME_DebugMsg("RME_Bag_DesactivateItem()",4);
		local ObjetExist, Quantity, Position = RME_Bag_ItemCount(LOCAL_ITEM_ID)
		if ObjetExist then
			RME_PLAYER_BAG[Position].Desactivated = SetDesactivated
		end
		if RME_BagFrame:IsShown() then
			RME_Bag_Update()
		end
	end
	
	-- Vrifie si les objets sont cohrents
	function RME_Bag_CheckItemRelationship()
		for i=1,table.getn(RME_PLAYER_BAG) do	
			local LOCAL_ITEM = RME_PLAYER_BAG[i]
			
			-- MISSION
			if LOCAL_ITEM.Type == "MISSION" then
				if not RME_MISSIONS_LIST[LOCAL_ITEM.Mission_LaunchID] then
					-- On desactive l'objet
					RME_PLAYER_BAG[i].Desactivated = SetDesactivated
					-- On demande la mission et on desactive l'objet en attendant
					RME_Comm_ASK_MISSION(LOCAL_ITEM.Mission_LaunchID,LOCAL_ITEM.FromPlayer)
				else
					-- On active l'objet
					RME_PLAYER_BAG[i].Desactivated = nil
				end
			
			
			elseif LOCAL_ITEM.Type == "USE_ITEM" or LOCAL_ITEM.Type =="USEONMOB" then
				
				if not RME_MISSIONS_LIST[LOCAL_ITEM.FromMissionId] then
					-- On desactive l'objet
					RME_PLAYER_BAG[i].Desactivated = SetDesactivated
					-- On demande la mission et on desactive l'objet en attendant
					RME_Comm_ASK_MISSION(LOCAL_ITEM.FromMissionId,LOCAL_ITEM.FromPlayer)
				else
					-- On active l'objet
					RME_PLAYER_BAG[i].Desactivated = nil
				end
			
			
			end
		end
		if RME_BagFrame:IsShown() then
			RME_Bag_Update()
		end
	end
	
--[[  ZONE MISSION LOADING ]]--
	
	-- Retourne la liste des missions disponibles dans la zone ainsi que la liste des interactions possible
	function RME_Load_Available_Mission(RealZone)
	
		RealZone = RME_Translate_To_UTF8(RealZone)
	
		RME_DebugMsg("RME_Load_Available_Mission("..tostring(RealZone)..")",4);
		--On reinit le tableau global comprenant les missions disponible dans la zone
		RME_AVAILABLE_MISSIONS_LIST = {} 
		-- On met a jour la liste des interactions possible dans cette zone
		RME_NPC_INTERACTION_LIST = {}
		
		local MISSION_ID, LOCAL_MISSION, missionfound
		missionfound = 0
		for MISSION_ID,LOCAL_MISSION in pairs(RME_MISSIONS_LIST) do -- LOCAL_MISSION est du style RME_CLASS_MISSION
			--RME_DebugMsg("RME_Load_Available_Mission("..tostring(RealZone).."): Check Mission Available ".. tostring(LOCAL_MISSION.Title) ,4);
			if LOCAL_MISSION:MissionCanBeTaken() and LOCAL_MISSION:NpcInZone(RealZone) then
				
				RME_AVAILABLE_MISSIONS_LIST[LOCAL_MISSION.ID]=LOCAL_MISSION
				missionfound = missionfound + 1
			end
		end
		RME_DebugMsg(tostring(missionfound).." new mission(s) found in this zone",3);		
		
		-- On boucle a prsent sur les mission du joueur
		for LOCAL_MISSION_ID,LOCAL_MISSION in pairs(RME_IN_PROGRESS_MISSIONS_LIST) do -- LOCAL_MISSION est du style RME_CLASS_MISSION
			--RME_DebugMsg("RME_Load_Available_Mission("..tostring(RealZone).."): Check Mission In Progress ".. tostring(LOCAL_MISSION.Title) ,4);
			LOCAL_MISSION:UpdateNpcInteraction()
		end

		RME_DebugMsg(tostring(RME_getn(RME_NPC_INTERACTION_LIST)).." NPC waiting player interaction in this zone." ,3);

		--Minimap et map
		RME_Minimap_Button_Update()
		
		-- On met a jour les infos
		if UnitExists("target") then
			RME_TARGET_INFO = RME_CLASS_TARGET("target")
			RME_TARGET_INFO:Update("target")
			RME_ACTUAL_RME_NPC_SELECTED = nil
			RME_ACTUAL_RME_NPC_SELECTED = RME_TARGET_INFO		
		
			-- La cible, rcement mise a jour, est un npc "RME" on stock ses donnes
			if RME_TARGET_INFO.IsAnNpcMission then
				RME_DebugMsg("RME_TARGET_INFO.IsAnNpcMission",2);
			end
		end		
	end
	
		
--[[  /ZONE MISSION LOADING ]]--	

--[[ LOAD MISSION/MISSION ]]--
	
	-- Genere dans dans TABLE_SORTANTE des class mission contenant les infos donne. Place ces missions dans TABLE_SORTANTE
	function RME_LOAD_MISSIONS(TABLE_SOURCE,TABLE_SORTANTE)
		RME_DebugMsg("RME_LOAD_MISSIONS("..tostring(TABLE_SOURCE)..","..tostring(TABLE_SORTANTE)..")",4);
		local MISSION_ID, LOCAL_MISSION_ENTRANTE, LOCAL_MISSION_SORTANTE
		for MISSION_ID,LOCAL_MISSION_ENTRANTE in pairs(TABLE_SOURCE) do -- LOCAL_MISSION_ENTRANTE est une simple table issue de la sauvegarde
			RME_DebugMsg("RME_LOAD_MISSIONS(#,#): Check Mission ".. tostring(MISSION_ID) ,4);
			if MISSION_ID then
				RME_DebugMsg("RME_LOAD_MISSIONS(#,#): Mission ".. tostring(MISSION_ID).." valid. Loading...",4);
				-- Ok on genere une mission
				LOCAL_MISSION_SORTANTE = RME_CLASS_MISSION(MISSION_ID)
				
				-- On charge les donnes de la mission
				LOCAL_MISSION_SORTANTE:Load(LOCAL_MISSION_ENTRANTE)
				LOCAL_MISSION_SORTANTE.NeedSave		=	nil
				
				-- Si la mission est correctement charge et active, on la place dans la TABLE_SORTANTE
				if LOCAL_MISSION_SORTANTE.Enabled then
					
					if (LOCAL_MISSION_SORTANTE.RMEVersion > 999) and (not LOCAL_MISSION_SORTANTE.Hash or LOCAL_MISSION_SORTANTE.Hash ~= LOCAL_MISSION_SORTANTE:ReturnHash()) then
						-- Pour la futur vrification du hashage des missions
						RME_DebugMsg("RME_LOAD_MISSIONS Mission "..tostring(LOCAL_MISSION_SORTANTE.Title)..": Mauvais Hash !",4);
							
					--[[elseif LOCAL_MISSION_SORTANTE.IsDeleted and LOCAL_MISSION_SORTANTE.CreateDate + 60*60*24*30 > time() then
						-- Mission supprime depuis 30j, il est donc raisonnable de penser que tout le monde est au courant a prsent
						-- on l'a supprime dfinitivement en ne la chargeant plus
						RME_DebugMsg("RME_LOAD_MISSIONS(#,#): Mission ".. tostring(MISSION_ID).." valid but deleted. Expiration > no loading",4);]]
					else
						RME_DebugMsg("RME_LOAD_MISSIONS(#,#): Mission ".. tostring(MISSION_ID).." valid. Loading complete !",4);
						TABLE_SORTANTE[MISSION_ID] = LOCAL_MISSION_SORTANTE
					end
				end
			end
		end
	end
	
	function RME_LOAD_MISSIONS_ID_ONLY(TABLE_SOURCE,TABLE_SORTANTE)
		RME_DebugMsg("RME_LOAD_MISSIONS("..tostring(TABLE_SOURCE)..","..tostring(TABLE_SORTANTE)..")",4);
		local MISSION_ID
		for MISSION_ID,LOCAL_MISSION_ENTRANTE in pairs(TABLE_SOURCE) do -- LOCAL_MISSION_ENTRANTE est une simple table issue de la sauvegarde
			if MISSION_ID then
				TABLE_SORTANTE[MISSION_ID] = MISSION_ID
			end
		end
	end
	
	function RME_LOAD_BAG(TABLE_SOURCE,TABLE_SORTANTE)
		RME_DebugMsg("RME_LOAD_BAG("..tostring(TABLE_SOURCE)..","..tostring(TABLE_SORTANTE)..")",4);
		local ITEM_ID, LOCAL_ITEM_ENTRANT, LOCAL_ITEM_SORTANT
		for ITEM_ID,LOCAL_ITEM_ENTRANT in pairs(TABLE_SOURCE) do -- LOCAL_ITEM_ENTRANT est une simple table issue de la sauvegarde
			--RME_DebugMsg("RME_LOAD_BAG(#,#): Check item ".. tostring(ITEM_ID) ,4);
			if ITEM_ID then
				--RME_DebugMsg("RME_LOAD_BAG(#,#): item ".. tostring(ITEM_ID).." valid. Loading...",4);
				-- Ok on genere un item
				LOCAL_ITEM_SORTANT = RME_CLASS_ITEM()
				-- On charge les donnes de l'item
				LOCAL_ITEM_SORTANT:Load(LOCAL_ITEM_ENTRANT)
				
				-- Si l'item est correctement charg et active, on la place dans la TABLE_SORTANTE
				if LOCAL_ITEM_SORTANT.Enabled then
					--RME_DebugMsg("RME_LOAD_BAG(#,#): item ".. tostring(MISSION_ID).." valid. Loading complete !",4);
					TABLE_SORTANTE[ITEM_ID] = LOCAL_ITEM_SORTANT
				end
			end
		end
	end
	
	function RME_LOAD_USERS(TABLE_SOURCE,TABLE_SORTANTE)
		RME_DebugMsg("RME_LOAD_USERS("..tostring(TABLE_SOURCE)..","..tostring(TABLE_SORTANTE)..")",4);
		local USER_ID, LOCAL_USER_ENTRANT, LOCAL_USER_SORTANT
		for USER_ID,LOCAL_USER_ENTRANT in pairs(TABLE_SOURCE) do -- LOCAL_USER_ENTRANT est une simple table issue de la sauvegarde
			--RME_DebugMsg("RME_LOAD_USERS(#,#): Check User ".. tostring(USER_ID) ,4);
			if USER_ID then
				--RME_DebugMsg("RME_LOAD_USERS(#,#): User ".. tostring(USER_ID).." valid. Loading...",4);
				-- Ok on genere une mission
				LOCAL_USER_SORTANT = RME_CLASS_USER(USER_ID)
				
				-- On charge les donnes de la mission
				LOCAL_USER_SORTANT:Load(LOCAL_USER_ENTRANT)
				
				-- Si la mission est correctement charge et active, on la place dans la TABLE_SORTANTE
				if LOCAL_USER_SORTANT.UniqueID then --vers 137: on ne prend plus en compte les users sans UniqueID
					--RME_DebugMsg("RME_LOAD_USERS(#,#): User ".. tostring(USER_ID).." valid. Loading complete !",4);
					TABLE_SORTANTE[USER_ID] = LOCAL_USER_SORTANT
				end
			end
		end
	end
--[[ /LOAD / SAVE MISSION ]]--

RME_DebugMsg("<LOAD>RME_Missions.lua"..RME_Return_ColoredText(" DONE !","ok"),3);
